from spaceone.api.file_manager.v1 import user_file_pb2, user_file_pb2_grpc
from spaceone.core.pygrpc import BaseAPI
from spaceone.file_manager.service.user_file_service import UserFileService


class UserFile(BaseAPI, user_file_pb2_grpc.UserFileServicer):
    pb2 = user_file_pb2
    pb2_grpc = user_file_pb2_grpc

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_file_svc = UserFileService(metadata)
        response: dict = user_file_svc.update(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_file_svc = UserFileService(metadata)
        user_file_svc.delete(params)
        return self.empty()

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_file_svc = UserFileService(metadata)
        response: dict = user_file_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_file_svc = UserFileService(metadata)
        response: dict = user_file_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        user_file_svc = UserFileService(metadata)
        response: dict = user_file_svc.stat(params)
        return self.dict_to_message(response)



import logging

from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)


class IdentityManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identity_connector: SpaceConnector = self.locator.get_connector(
            SpaceConnector, service="identity"
        )

    def check_workspace(self, workspace_id, domain_id):
        system_token = config.get_global("TOKEN")
        return self.identity_connector.dispatch(
            "Workspace.check",
            {"workspace_id": workspace_id, "domain_id": domain_id},
            token=system_token,
        )

    def get_project(self, project_id, domain_id):
        system_token = config.get_global("TOKEN")
        return  self.identity_connector.dispatch(
            "Project.get",
            {"project_id": project_id},x_domain_id=domain_id,
            token=system_token,
        )



from datetime import datetime
from typing import Union, List
from pydantic import BaseModel

from spaceone.core import utils
from spaceone.file_manager.model.file.request import ResourceGroup

__all__ = ["FileResponse", "FilesResponse"]


class FileResponse(BaseModel):
    file_id: Union[str, None] = None
    name: Union[str, None] = None
    download_url: Union[str, None] = None
    reference: Union[dict, None] = None
    tags: Union[dict, None] = None
    resource_group: Union[ResourceGroup, None] = None
    domain_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    project_id: Union[str, None] = None
    created_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        
        if data["resource_group"] == "SYSTEM":
            data["download_url"] = "/files/public/" + data["file_id"]
        elif data["resource_group"] == "DOMAIN":
            data["download_url"] = "/files/domain/" + data["file_id"]
        elif data["resource_group"] == "WORKSPACE":
            data["download_url"] = "/files/workspace/" + data["file_id"]
        elif data["resource_group"] == "PROJECT":
            data["download_url"] = "/files/project/" + data["file_id"]
        
        return data


class FilesResponse(BaseModel):
    results: List[FileResponse]
    total_count: int



import logging

from signal import raise_signal
from typing import Union

from spaceone.core import utils
from spaceone.core.service import *
from spaceone.file_manager.error.custom import *
from spaceone.file_manager.model.file.request import *
from spaceone.file_manager.model.file.response import *
from spaceone.file_manager.manager.file_manager import FileManager
from spaceone.file_manager.manager.file_connector_manager import FileConnectorManager
from spaceone.file_manager.manager.identity_manager import IdentityManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class FileService(BaseService):
    resource = "File"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.file_mgr = FileManager()
        self.identity_mgr = IdentityManager()

    @transaction(
        permission="file-manager:File.write",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @convert_model
    def add(self, params: FileAddRequest) -> Union[FileResponse, dict]:
        """Add file

        Args:
            params (FileAddRequest): {
                'name': 'str',              # required
                'reference': 'dict',
                'tags': 'dict',
                'resource_group': 'str',    # required
                'domain_id': 'str'          # injected from auth
                'workspace_id': 'str',      # injected from auth
                'project_id': 'str'         # injected from auth
            }

        Returns:
            FileResponse:
        """

        resource_group = params.resource_group
        
        if resource_group == "SYSTEM":
            params.domain_id = "*"
            params.workspace_id = "*"
        elif resource_group == "DOMAIN":
            params.workspace_id = "*"
        elif resource_group == "WORKSPACE" :
            self.identity_mgr.check_workspace(params.workspace_id, params.domain_id)
            params.project_id = "*"
        elif resource_group == "PROJECT":
            if not params.project_id:
                params.project_id = "*"
            else :
                self.identity_mgr.get_project(params.project_id, params.domain_id)


        file_vo = self.file_mgr.create_file(params.dict())
        return FileResponse(**file_vo.to_dict())

    @transaction(
        permission="file-manager:File.write",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @convert_model
    def update(self, params: FileUpdateRequest) -> Union[FileResponse, dict]:
        """Update file

        Args:
            params (FileUpdateRequest): {
                'file_id': 'str',           # required
                'reference': 'dict',
                'tags': 'dict',
                'download_url': 'str',
                'domain_id': 'str'          # injected from auth
                'workspace_id': 'str',      # injected from auth
                'project_id': 'str'         # injected from auth
            }

        Returns:
            FileResponse:
        """

        file_vo = self.file_mgr.get_file(
            params.file_id,
            params.domain_id,
            params.workspace_id,
            params.project_id,
            
        )

        file_vo = self.file_mgr.update_file_by_vo(
            params.dict(exclude_unset=True), file_vo
        )

        return FileResponse(**file_vo.to_dict())

    @transaction(
        permission="file-manager:File.write",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @convert_model
    def delete(self, params: FileDeleteRequest) -> None:
        """Delete file

        Args:
            params (FileDeleteRequest): {
                'file_id': 'str',           # required
                'domain_id': 'str'          # injected from auth
                'workspace_id': 'str',      # injected from auth
                'project_id': 'str'         # injected from auth
            }

        Returns:
            None:
        """

        file_vo = self.file_mgr.get_file(
            params.file_id,
            params.domain_id,
            params.workspace_id,
            params.project_id,
        )
        
        resource_group = file_vo["resource_group"]
        file_id = file_vo["file_id"]
        
        try:
            file_conn_mgr = FileConnectorManager()
            file_conn_mgr.delete_file(resource_group, file_id)
        except Exception as e:
            logging.error(f'[ERROR] Failed to delete file : {file_vo.name} ({file_vo.file_id})')
            raise ERROR_FILE_DELETE_FAILED(file_id=file_id)

        self.file_mgr.delete_file_by_vo(file_vo)

    @transaction(
        permission="file-manager:File.read",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )

    @convert_model
    def get(self, params: FileGetRequest) -> Union[FileResponse, dict]:
        """Get file

        Args:
            params (FileGetRequest): {
                'file_id': 'str',           # required
                'resource_group': 'str',
                'workspace_id': 'str',      # injected from auth
                'domain_id': 'str'          # injected from auth
                'project_id': 'str'         # injected from auth
                'user_projects': 'list'     # injected from auth
            }

        Returns:
            FileResponse:
        """

        resource_group = params.resource_group
        
        if resource_group == "SYSTEM":
            params.domain_id = "*"
            params.workspace_id = "*"
        elif resource_group == "DOMAIN":
            params.workspace_id = "*"
        elif resource_group == "WORKSPACE" :
            self.identity_mgr.check_workspace(params.workspace_id, params.domain_id)
            params.project_id = "*"
        elif resource_group == "PROJECT":
            if not params.project_id:
                params.project_id = "*"
            else :
                self.identity_mgr.get_project(params.project_id, params.domain_id)




        file_vo = self.file_mgr.get_file(
            params.file_id,
            params.domain_id,
            params.workspace_id,
            params.project_id,
        )

        return FileResponse(**file_vo.to_dict())

    @transaction(
        permission="file-manager:File.read",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @change_value_by_rule("APPEND", "domain_id", "*")
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @append_query_filter(
        [
            "file_id",
            "name",
            "resource_type",
            "resource_id",
            "domain_id",
            "workspace_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["file_id", "name"])
    @convert_model
    def list(self, params: FileSearchQueryRequest) -> Union[FilesResponse, dict]:
        """List files

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'file_id': 'str',
                'name': 'str',
                'resource_type': 'str',
                'resource_id': 'str',
                'domain_id': 'str',                             # injected from auth
                'workspace_id': 'str',                          # injected from auth
                'user_projects': 'list',                        # injected from auth
            }

        Returns:
            FilesResponse:
        """

        query = params.query or {}
        file_vos, total_count = self.file_mgr.list_files(query)
        files_info = [file_vo.to_dict() for file_vo in file_vos]

        return FilesResponse(results=files_info, total_count=total_count)

    @transaction(
        permission="file-manager:File.read",
        role_types=[
            "SYSTEM_ADMIN",
            "DOMAIN_ADMIN",
            "WORKSPACE_OWNER",
            "WORKSPACE_MEMBER",
        ],
    )
    @change_value_by_rule("APPEND", "domain_id", "*")
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @append_query_filter(["domain_id", "workspace_id", "user_projects"])
    @append_keyword_filter(["file_id", "name"])
    @convert_model
    def stat(self, params: FileStatQueryRequest) -> dict:
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
                'domain_id': 'str',                                     # injected from auth
                'workspace_id': 'str',                                  # injected from auth
            }

        Returns:
            dict:
        """

        query = params.query or {}
        return self.file_mgr.stat_files(query)



import logging
from typing import Union

from spaceone.core import utils
from spaceone.core.service import *
from spaceone.file_manager.error.custom import *
from spaceone.file_manager.model.user_file.request import *
from spaceone.file_manager.model.user_file.response import *
from spaceone.file_manager.manager.user_file_manager import UserFileManager
from spaceone.file_manager.manager.file_connector_manager import FileConnectorManager
from spaceone.file_manager.manager.identity_manager import IdentityManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class UserFileService(BaseService):
    resource = "UserFile"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_file_mgr = UserFileManager()
        self.identity_mgr = IdentityManager()

    @transaction(
        permission="file-manager:UserFile.write",
        role_types=["USER"],
    )
    @convert_model
    def add(self, params: UserFileAddRequest) -> Union[UserFileResponse, dict]:
        """Add file

        Args:
            params (UserFileAddRequest): {
                'name': 'str',              # required
                'reference': 'dict',
                'tags': 'dict',
                'domain_id': 'str',         # injected from auth
                'user_id': 'str',           # injected from auth
            }
        Returns:
            UserFileResponse:
        """

        user_file_vo = self.user_file_mgr.create_user_file(params.dict())
        return UserFileResponse(**user_file_vo.to_dict())

    @transaction(
        permission="file-manager:UserFile.write",
        role_types=[
            "USER",
        ],
    )
    @convert_model
    def update(self, params: UserFileUpdateRequest) -> Union[UserFileResponse, dict]:
        """Update file

        Args:
            params (FileUpdateRequest): {
                'file_id': 'str',           # required
                'reference': 'dict',
                'tags': 'dict',
                'download_url': 'str',
                'user_id': 'str'            # injected from auth
                'domain_id': 'str'          # injected from auth
            }

        Returns:
            UserFileResponse:
        """

        user_file_vo = self.user_file_mgr.get_user_file(
            params.file_id,
            params.domain_id, 
            params.user_id,
        )
        
        user_file_vo = self.user_file_mgr.update_user_file_by_vo(
            params.dict(exclude_unset=True), 
            user_file_vo
        )

        return UserFileResponse(**user_file_vo.to_dict())

    @transaction(
        permission="file-manager:UserFile.write",
        role_types=["USER"],
    )
    @convert_model
    def delete(self, params: UserFileDeleteRequest) -> None:
        """Delete file

        Args:
            params (FileDeleteRequest): {
                'file_id': 'str',           # required
                'user_id': 'str'            # injected from auth
                'domain_id': 'str'          # injected from auth
            }

        Returns:
            None:
        """

        user_file_vo = self.user_file_mgr.get_user_file(
            params.file_id,
            params.domain_id,
            params.user_id,
        )
        
        try:
            file_conn_mgr = FileConnectorManager()
            file_conn_mgr.delete_file("USER", user_file_vo.file_id)
        except Exception as e:
            _LOGGER.error(f"[delete] Failed to delete file: {user_file_vo.file_id}")
            raise ERROR_FILE_DELETE_FAILED(name=user_file_vo["download_url"])
        
        self.user_file_mgr.delete_user_file_by_vo(user_file_vo)

    @transaction(
        permission="file-manager:UserFile.read",
        role_types=["USER"],
    )
    @convert_model
    def get(self, params: UserFileGetRequest) -> Union[UserFileResponse, dict]:
        """Get file

        Args:
            params (FileGetRequest): {
                'file_id': 'str',           # required
                'user_id': 'str',           # injected from auth
                'domain_id': 'str'          # injected from auth
            }

        Returns:
            UserFileResponse:
        """

        user_file_vo = self.user_file_mgr.get_user_file(
            params.file_id, 
            params.domain_id, 
            params.user_id,
        )

        return UserFileResponse(**user_file_vo.to_dict())

    @transaction(
        permission="file-manager:UserFile.read",
        role_types=["USER"],
    )
    @append_query_filter(
        [
            "file_id",
            "name",
            "resource_type",
            "resource_id",
            "domain_id",
            "user_id",
        ]
    )
    @append_keyword_filter(["file_id", "name"])
    @convert_model
    def list(self, params: UserFileSearchQueryRequest) -> Union[UserFilesResponse, dict]:
        """List files

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'file_id': 'str',
                'name': 'str',
                'resource_type': 'str',
                'resource_id': 'str',
                'domain_id': 'str',                             # injected from auth
                'user_id': 'str',                               # injected from auth
            }

        Returns:
            UserFilesResponse:
        """

        query = params.query or {}
        user_file_vos, total_count = self.user_file_mgr.list_user_files(query)
        user_files_info = [user_file_vos.to_dict() for user_file_vo in user_file_vos]

        return UserFilesResponse(results=user_files_info, total_count=total_count)

    @transaction(
        permission="file-manager:UserFile.read",
        role_types=["USER"],
    )
    @append_query_filter(["domain_id", "user_id"])
    @append_keyword_filter(["file_id", "name"])
    @convert_model
    def stat(self, params: UserFileStatQueryRequest) -> dict:
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
                'domain_id': 'str',                                     # injected from auth
                'user_id': 'str',                                  # injected from auth
            }

        Returns:
            dict:
        """

        query = params.query or {}
        return self.user_file_mgr.stat_user_files(query)




from spaceone.core.error import *


class ERROR_NOT_AUTHORIZATION_USER(ERROR_BASE):
    _message = "Not authorization user. (user_id = {user_id}, domain_id = {domain_id})"

class  ERROR_NOT_SUPPORTED_RESOURCE_GROUP(ERROR_BASE):
    _message = "Not supported resource group. (resource_group = {resource_group})"

class ERROR_NOT_DEFINED_FILE_BACKEND(ERROR_BASE):
    _message = "File backend not defined. (backend = {backend})"

class ERROR_FILE_DOWNLOAD_URL_EXIST(ERROR_BASE):
    _message = "File download url is not exist. (file_id = {file_id}, name = {name})"


class ERROR_FILE_DOWNLOAD_FAILED(ERROR_BASE):
    _message = "File download failed. (name = {name})"
    
class ERROR_NOT_MATCH_USER_ID(ERROR_BASE):
    _message = "Not match user_id. (user_id = {user_id}, domain_id = {domain_id})"
    
class ERROR_FILE_UPLOAD_FAILED(ERROR_BASE):
    _message = "File upload failed. (name = {name})"
    
    
class ERROR_FILE_DELETE_FAILED(ERROR_BASE):
    _message = "File delete failed. (name = {name})"


import abc
from spaceone.core.connector import BaseConnector


class FileBaseConnector(BaseConnector):
    
    @abc.abstractmethod
    def check_file(self, resource_group:str, file_id:str) -> bool:
        pass

    @abc.abstractmethod
    def delete_file(self, resource_group:str, file_id:str ) -> None:
        pass

    @abc.abstractmethod
    def upload_file(self, resource_group:str, file_id:str, data:bytes) -> None:
        pass
    
    @abc.abstractmethod
    def download_file(self, resource_group:str, file_id:str ):
        pass
    


import logging
from mongoengine import QuerySet

from spaceone.core.manager import BaseManager
from spaceone.file_manager.model.user_file.database import UserFile

_LOGGER = logging.getLogger(__name__)


class UserFileManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user_file_model = UserFile

    def create_user_file(self, params: dict) -> UserFile:
        def _rollback(vo: UserFile) -> None:
            _LOGGER.info(f"[ROLLBACK] Delete user_file : {vo.name} ({vo.file_id})")
            vo.delete()

        user_file_vo: UserFile = self.user_file_model.create(params)
        self.transaction.add_rollback(_rollback, user_file_vo)

        return user_file_vo

    def update_user_file_by_vo(self, params: dict, user_file_vo: UserFile) -> UserFile:
        def _rollback(old_data: dict):
            _LOGGER.info(
                f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["user_file_id"]})'
            )
            user_file_vo.update(old_data)

        self.transaction.add_rollback(_rollback, user_file_vo.to_dict())

        return user_file_vo.update(params)

    @staticmethod
    def delete_user_file_by_vo(user_file_vo: UserFile) -> None:
        user_file_vo.delete()

    def get_user_file(
        self,
        file_id: str,
        domain_id: str,
        user_id: str,
    ) -> UserFile:

        return self.user_file_model.get(file_id=file_id, domain_id=domain_id, user_id=user_id)

    def filter_user_files(self, **conditions) -> QuerySet:
        return self.user_file_model.filter(**conditions)

    def list_user_files(self, query: dict) -> dict:
        return self.user_file_model.query(**query)

    def stat_user_files(self, query: dict) -> dict:
        return self.user_file_model.stat(**query)



from typing import Union, Literal
from pydantic import BaseModel

__all__ = [
    "UserFileAddRequest",
    "UserFileUpdateRequest",
    "UserFileDeleteRequest",
    "UserFileGetRequest",
    "UserFileSearchQueryRequest",
    "UserFileStatQueryRequest",
]

class UserFileAddRequest(BaseModel):
    name: str 
    domain_id: Union[str, None] = None
    user_id: Union[str, None] = None


class UserFileUpdateRequest(BaseModel):
    file_id: str
    reference: Union[dict, None] = None
    tags: Union[dict, None] = None
    domain_id: Union[str, None] = None
    user_id: Union[str, None] = None


class UserFileDeleteRequest(BaseModel):
    file_id: str
    domain_id: Union[str, None] = None
    user_id: Union[str, None] = None


class UserFileGetRequest(BaseModel):
    file_id: str
    domain_id: Union[list, str, None] = None
    user_id: Union[str, None] = None


class UserFileSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    file_id: Union[str, None] = None
    name: Union[str, None] = None
    resource_type: Union[str, None] = None
    resource_id: Union[str, None] = None
    domain_id: Union[list, str, None] = None
    user_id: Union[str, None] = None


class UserFileStatQueryRequest(BaseModel):
    query: dict
    domain_id: Union[list, str, None] = None
    user_id: Union[str, None] = None



from typing import Union, Literal
from pydantic import BaseModel


__all__ = [
    "FileAddRequest",
    "FileUpdateRequest",
    "FileDeleteRequest",
    "FileGetRequest",
    "FileSearchQueryRequest",
    "FileStatQueryRequest",
    "ResourceGroup",
]

ResourceGroup = Literal["SYSTEM", "DOMAIN", "WORKSPACE", "PROJECT"]


class FileAddRequest(BaseModel):
    name: str 
    resource_group: ResourceGroup
    domain_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    project_id: Union[str, None] = None


class FileUpdateRequest(BaseModel):
    file_id: str
    reference: Union[dict, None] = None
    tags: Union[dict, None] = None
    domain_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    project_id: Union[str, None] = None


class FileDeleteRequest(BaseModel):
    file_id: str
    domain_id: Union[str, None] = None
    workspace_id: Union[str, None] = None
    project_id: Union[str, None] = None


class FileGetRequest(BaseModel):
    file_id: str
    resource_group: ResourceGroup
    domain_id: Union[list, str, None] = None
    workspace_id: Union[list, str, None] = None
    project_id: Union[str, None] = None
    user_projects: Union[list, None] = None


class FileSearchQueryRequest(BaseModel):
    query: Union[dict, None] = None
    file_id: Union[str, None] = None
    name: Union[str, None] = None
    resource_type: Union[str, None] = None
    resource_id: Union[str, None] = None
    domain_id: Union[list, str, None] = None
    workspace_id: Union[list, str, None] = None
    project_id: Union[str, None] = None
    user_projects: Union[list, None] = None


class FileStatQueryRequest(BaseModel):
    query: dict
    domain_id: Union[list, str, None] = None
    workspace_id: Union[list, str, None] = None
    project_id: Union[str, None] = None



from spaceone.api.file_manager.v1 import file_pb2, file_pb2_grpc
from spaceone.core.pygrpc import BaseAPI
from spaceone.file_manager.service.file_service import FileService


class File(BaseAPI, file_pb2_grpc.FileServicer):
    pb2 = file_pb2
    pb2_grpc = file_pb2_grpc

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)
        file_svc = FileService(metadata)
        response: dict = file_svc.update(params)
        return self.dict_to_message(response)

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)
        file_svc = FileService(metadata)
        file_svc.delete(params)
        return self.empty()

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)
        file_svc = FileService(metadata)
        response: dict = file_svc.get(params)
        return self.dict_to_message(response)

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)
        file_svc = FileService(metadata)
        response: dict = file_svc.list(params)
        return self.dict_to_message(response)

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)
        file_svc = FileService(metadata)
        response: dict = file_svc.stat(params)
        return self.dict_to_message(response)



import logging

from spaceone.core import config, cache
from spaceone.core.manager import BaseManager
from spaceone.file_manager.error import *
from spaceone.file_manager.connector.file_base_connector import FileBaseConnector

_LOGGER = logging.getLogger(__name__)


class FileConnectorManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        backend = config.get_global("BACKEND", "FileConnectorManager")
        try:
            _LOGGER.debug(f"[FileConnectorManager] Create {backend}")
            self.file_conn: FileBaseConnector = self.locator.get_connector(backend)
        except Exception as e:
            _LOGGER.error(f"[FileConnectorManager] not defined backend {backend}")
            raise ERROR_NOT_DEFINED_FILE_BACKEND(backend=backend)

    def check_file(self, resource_group:str, file_id:str ):
        return self.file_conn.check_file(resource_group, file_id)

    def delete_file(self, resource_group:str, file_id:str ) -> None:
        self.file_conn.delete_file(resource_group, file_id)

    def upload_file(self, resource_group:str, file_id:str , file_binary: bytes) -> None:
        self.file_conn.upload_file( resource_group, file_id, file_binary)  

    def download_file(self, resource_group:str, file_id:str ) :
        return self.file_conn.download_file(resource_group, file_id)




import logging

from fastapi import Request, Depends, File, UploadFile
from fastapi.responses import StreamingResponse
from fastapi.concurrency import run_in_threadpool
from fastapi_utils.cbv import cbv
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials
from fastapi_utils.inferring_router import InferringRouter


from spaceone.core import utils
from spaceone.core.fastapi.api import BaseAPI, exception_handler
from spaceone.file_manager.manager.file_connector_manager import FileConnectorManager
from spaceone.file_manager.model import user_file
from spaceone.file_manager.service.user_file_service import UserFileService
from spaceone.file_manager.error import *

_LOGGER = logging.getLogger(__name__)
_AUTH_SCHEME = HTTPBearer(auto_error=False)

router = InferringRouter(include_in_schema=False)

@cbv(router)
class UserFiles(BaseAPI):
    token: HTTPAuthorizationCredentials = Depends(_AUTH_SCHEME)
    service = "file-manager"
    
    @router.post("/user/upload")
    @exception_handler
    async def upload_user_file(self, request: Request, file: UploadFile = File(...)):
        
        metadata = {
            "token": self.token.credentials,
        }
        params = {
            "name": file.filename,
        }
        
        user_file_info = await self.upload_file(metadata, params, file)
        return user_file_info

    @router.get("/user/{file_id}")
    @exception_handler
    async def download_user_file(self, file_id:str, token:str,  request: Request):

        metadata = {
            "token": token,
        }
        params = {
            "file_id": file_id,
        }
        
        return await self.download_file(metadata, params)
    
    async def upload_file(self, metadata, params, file) :
        
        try:
            user_file_svc = UserFileService(metadata)
            user_file_info: dict = user_file_svc.add(params)

            resource_group = "USER"
            file_id = user_file_info["file_id"]

            file_conn_mgr = FileConnectorManager()
            await run_in_threadpool(file_conn_mgr.upload_file, resource_group, file_id, await file.read())
        except Exception as e:
            user_file_svc.delete({"file_id":file_id})
            raise ERROR_FILE_UPLOAD_FAILED(name=user_file_info["name"])

        return user_file_info

    async def download_file(self, metadata, params) -> StreamingResponse:

        user_file_svc = UserFileService(metadata)
        user_file_info: dict = user_file_svc.get(params)
        
        resource_group = "USER"
        file_id = user_file_info["file_id"]

        try:
            file_conn_mgr = FileConnectorManager()
            obj = await run_in_threadpool(file_conn_mgr.download_file, resource_group, file_id)
            if not obj:
                raise ERROR_FILE_DOWNLOAD_FAILED(name=user_file_info["name"])
            
        except Exception as e:
            raise ERROR_FILE_DOWNLOAD_FAILED(name=user_file_info["name"])

        return StreamingResponse(
            content=obj["Body"],
            media_type="application/octet-stream",
            headers={"Content-Disposition": f"attachment; filename={user_file_info['name']}", "content-length": str(obj["ContentLength"])}
        )


import logging

from fastapi import Request, Depends, File, UploadFile
from fastapi.responses import StreamingResponse
from fastapi.concurrency import run_in_threadpool
from fastapi_utils.cbv import cbv
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials
from fastapi_utils.inferring_router import InferringRouter


from spaceone.core import utils
from spaceone.core.fastapi.api import BaseAPI, exception_handler
from spaceone.file_manager.manager.file_connector_manager import FileConnectorManager
from spaceone.file_manager.service.file_service import FileService
from spaceone.file_manager.error import *

_LOGGER = logging.getLogger(__name__)
_AUTH_SCHEME = HTTPBearer(auto_error=False)

router = InferringRouter(include_in_schema=False)


@cbv(router)
class Files(BaseAPI):
    token: HTTPAuthorizationCredentials = Depends(_AUTH_SCHEME)
    service = "file-manager"

    @router.post("/public/upload")
    @exception_handler
    async def upload_public_file(self, request: Request, file: UploadFile = File(...)):
        
        metadata = {
            "token": self.token.credentials,
        }
        params = {
            "name": file.filename,
            "resource_group": "SYSTEM",
        }

        file_info = await self.upload_file(metadata, params, file)
        return file_info

    @router.get("/public/{file_id}")
    @exception_handler
    async def download_public_file(self, request: Request, file_id: str, token:str):

        metadata = {
            "token": token,
        }
        params = {
            "file_id": file_id,
            "resource_group": "SYSTEM",
        }

        return await self.download_file(metadata, params)

    @router.post("/domain/upload")
    @exception_handler
    async def upload_domain_file(self, request: Request, file: UploadFile = File(...)):

        metadata = {
            "token": self.token.credentials,
        }
        params = {
            "name": file.filename,
            "resource_group": "DOMAIN",
        }

        file_info = await self.upload_file(metadata, params, file)
        return file_info

    @router.get("/domain/{file_id}")
    @exception_handler
    async def download_domain_file(self, file_id:str, token:str, request: Request) -> StreamingResponse:

        metadata = {
            "token": token,
        }
        params = {
            "file_id": file_id,
            "resource_group": "DOMAIN",
        }
        
        return await self.download_file(metadata, params)

    @router.post("/workspace/upload")
    @exception_handler
    async def upload_workspace_file(self, request: Request, file: UploadFile = File(...)):
        
        metadata = {
            "token": self.token.credentials,
        }
        params = {
            "name": file.filename,
            "resource_group": "WORKSPACE",
        }
        file_info = await self.upload_file(metadata, params, file)
        return file_info

    @router.get("/workspace/{file_id}")
    @exception_handler
    async def download_workspace_file(self, file_id:str, token:str, request: Request):

        metadata = {
            "token": token,
        }
        params = {
            "file_id": file_id,
            "resource_group": "WORKSPACE",
        }

        return await self.download_file(metadata, params)


    @router.post("/project/upload")
    @exception_handler
    async def upload_project_file(self, project_id:str, request: Request, file: UploadFile = File(...)):
        
        metadata = {
            "token": self.token.credentials,
        }
        params = {
            "name": file.filename,
            "resource_group": "PROJECT",
        }
        
        
        if project_id:
            params["project_id"] = project_id
        else:
            params["project_id"] = "*"
        
        file_info = await self.upload_file(metadata, params, file)
        return file_info

    @router.get("/project/{file_id}")
    @exception_handler
    async def download_project_file(self, file_id:str, token:str, request: Request)-> StreamingResponse:

        metadata = {
            "token": token,
        }
        params = {
            "file_id": file_id,
            "resource_group": "WORKSPACE",
        }

        return await self.download_file(metadata, params)

    async def upload_file(self, metadata, params, file) :
        
        try:

            file_svc = FileService(metadata)
            file_info: dict = file_svc.add(params)

            resource_group = file_info["resource_group"]
            file_id = file_info["file_id"]

            file_conn_mgr = FileConnectorManager()
            await run_in_threadpool(file_conn_mgr.upload_file, resource_group, file_id, await file.read())
        except Exception as e:
            file_svc.delete({"file_id":file_id})
            raise ERROR_FILE_UPLOAD_FAILED(name=file_info["name"])

        return file_info

    async def download_file(self, metadata, params) -> StreamingResponse:

        file_svc = FileService(metadata)
        file_info: dict = file_svc.get(params)
        
        resource_group = file_info["resource_group"]
        file_id = file_info["file_id"]

        try:
            file_conn_mgr = FileConnectorManager()
            obj = await run_in_threadpool(file_conn_mgr.download_file, resource_group, file_id)
            if not obj:
                raise ERROR_FILE_DOWNLOAD_FAILED(name=file_info["name"])
            
        except Exception as e:
            raise ERROR_FILE_DOWNLOAD_FAILED(name=file_info["name"])

        return StreamingResponse(
            content=obj["Body"],
            media_type="application/octet-stream",
            headers={"Content-Disposition": f"attachment; filename={file_info['name']}", "content-length": str(obj["ContentLength"])}
        )



from spaceone.core.pygrpc.server import GRPCServer
from spaceone.file_manager.interface.grpc.file import File
from spaceone.file_manager.interface.grpc.user_file import UserFile

_all_ = ["app"]

app = GRPCServer()
app.add_service(File)
app.add_service(UserFile)



ROUTER = [
    {
        "router_path": "spaceone.file_manager.interface.rest.file:router",
        "router_options": {
            "prefix": "/files",
        },
    },
    {
        "router_path": "spaceone.file_manager.interface.rest.user_file:router",
        "router_options": {
            "prefix": "/files",
        },
    },
]



from datetime import datetime
from typing import Union, List
from pydantic import BaseModel

from spaceone.core import utils

__all__ = ["UserFileResponse", "UserFilesResponse"]


class UserFileResponse(BaseModel):
    file_id: Union[str, None] = None
    name: Union[str, None] = None
    reference: Union[dict, None] = None
    tags: Union[dict, None] = None
    domain_id: Union[str, None] = None
    user_id: Union[str, None] = None
    created_at: Union[datetime, None] = None

    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = utils.datetime_to_iso8601(data["created_at"])
        data["download_url"] = "/files/user/" + data["file_id"]
        return data


class UserFilesResponse(BaseModel):
    results: List[UserFileResponse]
    total_count: int
